let grid_size = 16;  // grid size is 16x16 : 256 colors in palete
let canvas_size = 256;
let colors = [];
let sorted_colors = [];
let w;

function rgb2hsl(rgb_color) {
  let r = rgb_color.r/255;
  let g = rgb_color.g/255;
  let b = rgb_color.b/255;
  let Cmax = Math.max(r, g, b);
  let Cmin = Math.min(r, g, b);

  let delta = Cmax - Cmin;
  let h; // in degrees

  if (delta == 0) h = 0;
  else if (Cmax == r) h = 60*((g-b)/delta % 6);
  else if (Cmax == g) h = 60*((b-r)/delta + 2);
  else if (Cmax == b) h = 60*((r-g)/delta + 4);

  let l = (Cmax + Cmin)/2;
  let s;

  if (delta == 0) s = 0;
  else s = delta/(1-Math.abs(2*l-1));

  return {h: h, s: s, l: l};
}

let subSort = (arr, i, n, sortFx) => [].concat(...arr.slice(0, i), ...arr.slice(i, i + n).sort(sortFx), ...arr.slice(i + n, arr.length));

function setup() {
  w = canvas_size / grid_size;
  createCanvas(2*canvas_size + w + 1, canvas_size + 1);
  colorMode(RGB);
  noLoop();

  for ( let c = 0; c < 256; c++ ) {
    colors[c] = { r: random(0, 256),
                  g: random(0, 256),
                  b: random(0, 256)};
  }

  sorted_colors = colors.slice();
  // Sort functions to choose from:
  let saturation_sort = function(a, b) { return rgb2hsl(b).s - rgb2hsl(a).s; };
  let hue_sort = function(a, b) { return rgb2hsl(b).h - rgb2hsl(a).h; };
  let lightness_sort = function(a, b) { return rgb2hsl(b).l - rgb2hsl(a).l; };
  let brightness_sort = function(a, b) { return Math.sqrt(b.r*b.r + b.g*b.g + b.b*b.b) - Math.sqrt(a.r*a.r + a.g*a.g + a.b*a.b); };

  // Sort proceess
  sorted_colors.sort(hue_sort);
  for ( let i = 0; i < grid_size; i++ )
    sorted_colors = subSort(sorted_colors, i*grid_size, grid_size, brightness_sort);
}

function draw() {
  background(0);

  //noStroke();
  // unsorted colors
  for (let i = 0; i < grid_size; i++) {
    for (let j = 0; j < grid_size; j++) {
      let c = colors[i*grid_size + j];
      fill(c.r, c.g, c.b);
      rect(j*w, i*w, w, w);
    }
  }

  // sorted sorted
  for (let i = 0; i < grid_size; i++) {
    for (let j = 0; j < grid_size; j++) {
      let c = sorted_colors[i*grid_size + j];
      fill(c.r, c.g, c.b);
      rect(j*w + w*(grid_size+1), i*w, w, w);
    }
  }
}
